module HooksManager
    class Placeholders
        include Redmine::I18n

        @@instance = nil
        @@placeholders = {}

        def self.[](hook)
            @@instance ||= new
            @@placeholders[hook]
        end

        def self.each(&block)
            @@instance ||= new
            @@placeholders.each(&block)
        end

        def self.sort(&block)
            @@instance ||= new
            @@placeholders.sort(&block)
        end

        def self.tree(top = :page, all = false)
            tree = {}
            @@instance ||= new
            @@placeholders.each do |hook, placeholder|
                if placeholder[:basic] || all
                    if top == :location
                        first = l("hook_location_#{placeholder[:location] || :other}")
                        second = l("hook_page_#{placeholder[:page] || :other}")
                    else
                        first = l("hook_page_#{placeholder[:page] || :other}")
                        second = l("hook_location_#{placeholder[:location] || :other}")
                    end
                    tree[first] ||= {}
                    tree[first][second] ||= {}
                    tree[first][second][placeholder[:hook]] = placeholder
                end
            end
            tree
        end

    protected

        def initialize
            @@placeholders = YAML.load_file("#{File.dirname(__FILE__)}/../config/placeholders.yml")
            Redmine::Plugin.registered_plugins.each do |id, plugin|
                if Redmine::Plugin.respond_to?(:directory)
                    plugin_config = File.join(Redmine::Plugin.directory, id.to_s, 'config/placeholders.yml')
                else
                    plugin_config = File.join(Rails.root, 'vendor/plugins', id.to_s, 'config/placeholders.yml')
                end
                if File.exist?(plugin_config)
                    @@placeholders.merge!(YAML.load_file(plugin_config))
                end
            end
            @@placeholders.symbolize_keys!
            @@placeholders.each do |hook, placeholder|
                placeholder[:description] = l("description_#{hook}", :default => '')
                placeholder[:hook] = hook
                placeholder.symbolize_keys!
            end
            current = Redmine::VERSION.to_a
            @@placeholders.reject! do |hook, placeholder|
                if placeholder.has_key?(:requires)
                    compare_versions(placeholder[:requires].to_s, current) > 0
                else
                    false
                end
            end
        end

        # A copy of Redmine::Plugin.compare_versions
        def compare_versions(required, current)
            required = required.split('.').map(&:to_i)
            required <=> current.slice(0, required.size)
        end

    end
end
